## 观察者模式
> 在对象之间定义了一对多的依赖，这样一来，当一个对象改变状态，依赖它的对象会收到通知并自动更新。
  其实就是发布订阅模式，发布者发布信息，订阅者获取信息，订阅了就能收到信息，没订阅就收不到信息。
> 四个角色
> - 抽象被观察者角色
> - 抽象观察者角色
> - 具体被观察者角色
> - 具体观察者角色

### 举例子(如：微信公众号)

> 定义一个抽象被观察者接口，声明了添加、删除、通知观察者方法。

```java
interface Observerable {
    void registerObserver(Observer o);
    void removeObserver(Observer o);
    void notifyObserver();
}
```

> 定义一个抽象观察者接口，定义了一个update()方法，当被观察者调用notifyObservers()方法时，观察者的update()方法会被回调。

```java
interface Observer {
    void update(String message);
}
```

> 定义被观察者，实现三个接口，并且list保存注册的Observer

```java
class WechatObserver implements Observerable {

    private List<Observer> list;
    private String message;

    public WechatObserver() {
        list = new ArrayList<Observer>();
    }

    @Override
    public void registerObserver(Observer o) {
        list.add(o);
    }

    @Override
    public void removeObserver(Observer o) {
        list.remove(o);
    }

    @Override
    public void notifyObserver() {
        for (Observer observer : list) {
            observer.update(message);
        }
    }

    public void setMessage(String message) {
        this.message = message;
        System.out.println("微信服务更新消息：" + message);
        // 消息更新，通知所有观察者
        notifyObserver();
    }
}
```
> 定义具体观察者，User

```java
class User implements Observer {

    private String name;
    private String message;

    public User(String name) {
        this.name = name;
    }


    @Override
    public void update(String message) {
        this.message = message;
        read();
    }

    void read() {
        System.out.println(name+ " 收到推送消息：" + message);
    }
}
```

### 测试

```java
public class ObserverMode {
    public static void main(String[] args) {
        WechatObserver wechatObserver = new WechatObserver();
        User maifeng = new User("Maifeng");
        User xiaofeng = new User("Xiaofeng");
        User fengfeng = new User("Fengfeng");

        wechatObserver.registerObserver(maifeng);
        wechatObserver.registerObserver(xiaofeng);
        wechatObserver.registerObserver(fengfeng);
        wechatObserver.setMessage("PHP是世界上最好的语言...");

        System.out.println("---------------");

        wechatObserver.removeObserver(fengfeng);
        wechatObserver.setMessage("JAVA是世界上最好的语言...");
    }
}
```